@@ -85,6 +85,10 @@ private slots:
     void tiled();
 
     void readRgba64();
+    void readGray16();
+
+    void colorSpace_data();
+    void colorSpace();
 
 private:
     QString prefix;
@@ -168,6 +172,7 @@ void tst_qtiff::readImage_data()
     QTest::newRow("tiled_oddsize_grayscale") << QString("tiled_oddsize_grayscale.tiff") << QSize(59, 71);
     QTest::newRow("tiled_oddsize_mono") << QString("tiled_oddsize_mono.tiff") << QSize(59, 71);
     QTest::newRow("16bpc") << QString("16bpc.tiff") << QSize(64, 46);
+    QTest::newRow("gray16") << QString("gray16.tiff") << QSize(64, 46);
 }
 
 void tst_qtiff::readImage()
@@ -178,9 +183,13 @@ void tst_qtiff::readImage()
     QString path = prefix + fileName;
     QImageReader reader(path);
     QVERIFY(reader.canRead());
+    QImage::Format headerFormat = reader.imageFormat();
+    QSize headerSize = reader.size();
     QImage image = reader.read();
     QVERIFY(!image.isNull());
     QCOMPARE(image.size(), size);
+    QCOMPARE(image.size(), headerSize);
+    QCOMPARE(image.format(), headerFormat);
 }
 
 void tst_qtiff::readCorruptImage_data()
@@ -386,7 +395,8 @@ void tst_qtiff::readWriteNonDestructive_data()
     QTest::newRow("tiff indexed") << QImage::Format_Indexed8 << QImage::Format_Indexed8 << QImageIOHandler::TransformationMirror;
     QTest::newRow("tiff argb32pm") << QImage::Format_ARGB32_Premultiplied << QImage::Format_ARGB32_Premultiplied << QImageIOHandler::TransformationRotate90;
     QTest::newRow("tiff rgb32") << QImage::Format_RGB32 << QImage::Format_RGB32 << QImageIOHandler::TransformationRotate270;
-    QTest::newRow("tiff grayscale") << QImage::Format_Grayscale8 << QImage::Format_Grayscale8 << QImageIOHandler::TransformationFlip;
+    QTest::newRow("tiff grayscale8") << QImage::Format_Grayscale8 << QImage::Format_Grayscale8 << QImageIOHandler::TransformationFlip;
+    QTest::newRow("tiff grayscale16") << QImage::Format_Grayscale16 << QImage::Format_Grayscale16 << QImageIOHandler::TransformationMirror;
     QTest::newRow("tiff rgb64") << QImage::Format_RGBX64 << QImage::Format_RGBX64 << QImageIOHandler::TransformationNone;
     QTest::newRow("tiff rgba64") << QImage::Format_RGBA64 << QImage::Format_RGBA64 << QImageIOHandler::TransformationRotate90;
     QTest::newRow("tiff rgba64pm") << QImage::Format_RGBA64_Premultiplied << QImage::Format_RGBA64_Premultiplied << QImageIOHandler::TransformationNone;
@@ -492,7 +502,7 @@ void tst_qtiff::supportsOption()
         allOptions.remove(QImageIOHandler::ImageOption(options.at(i)));
     }
 
-    foreach (QImageIOHandler::ImageOption option, allOptions)
+    for (QImageIOHandler::ImageOption option : qAsConst(allOptions))
         QVERIFY(!writer.supportsOption(option));
 }
 
@@ -609,5 +619,51 @@ void tst_qtiff::readRgba64()
     QCOMPARE(image.format(), QImage::Format_RGBX64);
 }
 
+void tst_qtiff::readGray16()
+{
+    QString path = prefix + QString("gray16.tiff");
+    QImageReader reader(path);
+    QVERIFY(reader.canRead());
+    QCOMPARE(reader.imageFormat(), QImage::Format_Grayscale16);
+    QImage image = reader.read();
+    QVERIFY(!image.isNull());
+    QCOMPARE(image.format(), QImage::Format_Grayscale16);
+}
+
+void tst_qtiff::colorSpace_data()
+{
+    QTest::addColumn<decltype(QColorSpace::SRgb)>("namedColorSpace");
+
+    QTest::newRow("sRGB")         << QColorSpace::SRgb;
+    QTest::newRow("sRGB(linear)") << QColorSpace::SRgbLinear;
+    QTest::newRow("AdobeRGB")     << QColorSpace::AdobeRgb;
+    QTest::newRow("DisplayP3")    << QColorSpace::DisplayP3;
+    QTest::newRow("ProPhotoRgb")  << QColorSpace::ProPhotoRgb;
+}
+
+void tst_qtiff::colorSpace()
+{
+    QFETCH(decltype(QColorSpace::SRgb), namedColorSpace);
+
+    QImage image(prefix + "colorful.bmp");
+    QVERIFY(!image.isNull());
+
+    image.setColorSpace(namedColorSpace);
+
+    QByteArray output;
+    QBuffer buf(&output);
+    QVERIFY(buf.open(QIODevice::WriteOnly));
+    QImageWriter writer(&buf, "tiff");
+    writer.write(image);
+    buf.close();
+
+    QVERIFY(buf.open(QIODevice::ReadOnly));
+    QImageReader reader(&buf);
+    QImage image2 = reader.read();
+
+    QCOMPARE(image2.colorSpace(), namedColorSpace);
+    QCOMPARE(image2, image);
+}
+
 QTEST_MAIN(tst_qtiff)
 #include "tst_qtiff.moc"
